@@ -17,7 +17,12 @@ spec :: Spec
 spec = do
   describe "Data Version instance" $
     it "allows obtaining a Version constructor" $
-      dataTypeName (dataTypeOf (Version [1,2,3] [])) `shouldBe` "Data.Version.Version"
+      dataTypeName (dataTypeOf (Version [1,2,3] [])) `shouldBe`
+#if __GLASGOW_HASKELL__ >= 801
+        "Version"
+#else
+        "Data.Version.Version"
+#endif
 
 #if MIN_VERSION_base(4,7,0)
   describe "IsList Version instance" $
